package com.threed.jpct.benchxl;

import java.io.BufferedInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import android.content.res.Resources;

import com.threed.jpct.FrameBuffer;
import com.threed.jpct.IPaintListener;
import com.threed.jpct.Texture;
import com.threed.jpct.TextureManager;
import com.threed.jpct.World;
import com.threed.jpct.util.MemoryHelper;

public abstract class AbstractBenchmark implements IPaintListener {

	private static final long serialVersionUID = 1L;
	protected boolean running = false;
	protected int frames = 0;
	protected long startTime = 0;
	protected long maxTime = 0;
	protected World world = null;
	protected Resources ress = null;
	protected FrameBuffer buffer = null;
	protected TextureManager texMan = TextureManager.getInstance();
	protected boolean done = false;
	protected long runTime = 0;
	protected BenchmarkResult res = null;

	protected long fps = 0;

	public AbstractBenchmark(Resources res, World world, FrameBuffer buffer, long maxTime) {
		this.world = world;
		this.buffer = buffer;
		this.maxTime = maxTime;
		this.ress = res;
	}

	public void finishedPainting() {
		if (!done) {
			if (!running) {
				MemoryHelper.compact();
				try {
					Thread.sleep(700);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				startTime = System.currentTimeMillis();
			} else {
				frames++;
			}
			running = true;
			if (maxTime != -1 && (runTime = System.currentTimeMillis() - startTime) >= maxTime) {
				done = true;
				dispose();
			}
		}
	}

	public abstract Texture getDescription();
	
	public abstract Texture getInfoTexture();

	public void startPainting() {
	}

	public void init() {
		world.getCamera().getBack().setIdentity();
		world.getCamera().setPosition(0, 0, 0);
		buffer.setPaintListener(this);
	}

	public abstract boolean paint();

	protected List<Float> readFloats(InputStream is) {
		int byt = 0;
		is=new BufferedInputStream(is, 256);
		List<Float> floats = new ArrayList<Float>();
		try {
			do {
				byt = is.read();
				if (byt != -1) {
					float ret = Float.intBitsToFloat((int) (((byt & 0xFF) << 24) + ((is.read() & 0xFF) << 16) + ((is.read() & 0xFF) << 8) + (is.read() & 0xFF)));
					floats.add(ret);
				}
			} while (byt != -1);
			is.close();
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
		return floats;
	}
	
	public void dispose() {
		done = true;
		res = new BenchmarkResult();
		res.frames = frames;
		res.pixels = frames * (buffer.getWidth() * buffer.getHeight());
		res.time = runTime;
		buffer.setPaintListener(null);
	}

	public boolean done() {
		return done;
	}

	public BenchmarkResult getResult() {
		return res;
	}
}
