package com.threed.jpct.benchxl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Iterator;

import javax.microedition.khronos.egl.EGL10;
import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.egl.EGLDisplay;
import javax.microedition.khronos.opengles.GL10;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.content.res.Resources;
import android.net.Uri;
import android.opengl.GLSurfaceView;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.PowerManager;
import android.util.Log;
import android.view.MotionEvent;
import android.view.Window;
import android.view.WindowManager.LayoutParams;
import android.widget.Toast;

import com.threed.jpct.Config;
import com.threed.jpct.FrameBuffer;
import com.threed.jpct.Logger;
import com.threed.jpct.RGBColor;
import com.threed.jpct.Texture;
import com.threed.jpct.TextureManager;
import com.threed.jpct.World;
import com.threed.jpct.benchxl.parts.ClothSimulation;
import com.threed.jpct.benchxl.parts.DoubleDragon;
import com.threed.jpct.benchxl.parts.ParticleSystem;
import com.threed.jpct.benchxl.parts.SkeletalAnimations;
import com.threed.jpct.benchxl.parts.Terrain;
import com.threed.jpct.util.MemoryHelper;

public class An3DBenchXL extends Activity {

	private GLSurfaceView mGLView;
	private MyRenderer renderer = null;
	private RGBColor color = new RGBColor(0, 0, 0);

	private long defaultTimeLong = 25000;
	private PowerManager.WakeLock wl = null;
	private String text = null;
	private String resolution = null;

	private boolean loaded = false;
	private boolean running = false;
	private boolean bit32 = true;
	private boolean vbo = true;

	private int width = 0;
	private int height = 0;

	private String url = "http://www.jpct.de/an3dbenchxl/upload.php?data=";

	private Handler myHandler = new Handler();

	private static boolean gl20 = false;

	final Runnable dialogOpener = new Runnable() {
		public void run() {
			showDialog(An3DBenchXL.DIALOG_RESULTS_ID);
		}
	};

	private static final int DIALOG_RESULTS_ID = 0;

	protected Dialog onCreateDialog(int id) {
		Dialog dialog;
		switch (id) {
		case DIALOG_RESULTS_ID:
			dialog = new AlertDialog.Builder(An3DBenchXL.this).setTitle("Results " + resolution).setMessage(text).setNegativeButton("Exit", new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog, int whichButton) {
					dialog.dismiss();
					An3DBenchXL.this.finish();
				}
			}).setPositiveButton("Upload results", new DialogInterface.OnClickListener() {
				public void onClick(DialogInterface dialog, int whichButton) {
					try {
						Intent myIntent = new Intent(Intent.ACTION_VIEW, Uri.parse(url));
						startActivity(myIntent);
					} catch (Exception e) {
						Logger.log(e);
						Toast.makeText(An3DBenchXL.this.getBaseContext(), "Unable to open browser!", Toast.LENGTH_LONG);
					}
					dialog.dismiss();
					An3DBenchXL.this.finish();
				}
			}).create();
			break;
		default:
			dialog = null;
		}
		return dialog;
	}

	protected void onCreate(Bundle savedInstanceState) {

		Config.glDebugLevel = 0; // Change to 1 to enable gl call output

		requestWindowFeature(Window.FEATURE_NO_TITLE);
		getWindow().setFlags(LayoutParams.FLAG_FULLSCREEN, LayoutParams.FLAG_FULLSCREEN);

		super.onCreate(savedInstanceState);
		init();
	}

	private void init() {

		mGLView = new GLSurfaceView(getApplication());

		if (gl20) {
			Toast.makeText(getApplication(), "Using OpenGL ES 2.0!", Toast.LENGTH_SHORT).show();
			mGLView.setEGLContextClientVersion(2);
		} else {
			Toast.makeText(getApplication(), "Using OpenGL ES 1.1!", Toast.LENGTH_SHORT).show();
			mGLView.setEGLConfigChooser(new GLSurfaceView.EGLConfigChooser() {
				public EGLConfig chooseConfig(EGL10 egl, EGLDisplay display) {
					int[] attributes = new int[] { EGL10.EGL_DEPTH_SIZE, 16, EGL10.EGL_NONE };
					EGLConfig[] configs = new EGLConfig[1];
					int[] result = new int[1];
					egl.eglChooseConfig(display, attributes, configs, 1, result);
					return configs[0];
				}
			});
		}

		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
		renderer = new MyRenderer();
		mGLView.setRenderer(renderer);
		setContentView(mGLView);

		PowerManager pm = (PowerManager) getSystemService(Context.POWER_SERVICE);
		wl = pm.newWakeLock(PowerManager.SCREEN_BRIGHT_WAKE_LOCK | PowerManager.ON_AFTER_RELEASE, "wtf?");
		wl.acquire();
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
	}

	@Override
	protected void onPause() {
		super.onPause();
		if (mGLView != null) {
			mGLView.onPause();
		}
		if (loaded) {
			if (renderer != null) {
				renderer.stop();
			}
			System.exit(0);
		}
	}

	@Override
	protected void onResume() {
		super.onResume();
		if (mGLView != null) {
			mGLView.onResume();
		}
	}

	protected void onStop() {
		super.onStop();
		if (wl != null) {
			wl.release();
		}
		if (renderer != null) {
			renderer.stop();
		}
	}

	protected boolean isFullscreenOpaque() {
		return true;
	}

	public boolean onTouchEvent(MotionEvent event) {
		if (!running) {
			float x = event.getX();
			float y = event.getY();

			int xa = (width / 2) - 128;
			int xe = xa + 256;

			int ya = (height / 2) - 128;
			int ye = ya + 256;

			if (x >= xa && x <= xe && y >= ya && y <= ye) {
				running = true;
				Logger.log("Starting benchmark with " + (bit32 ? "32bit textures" : "16bit textures") + " and " + (vbo ? "with" : "without") + " VBO support!");
				return true;
			}
		}
		// Silence the touch events a little...
		try {
			Thread.sleep(20);
		} catch (InterruptedException e) {
		}

		return super.onTouchEvent(event);
	}

	class MyRenderer implements GLSurfaceView.Renderer {
		private static final long serialVersionUID = 1L;
		private FrameBuffer fb = null;
		private World world = null;

		private boolean init = false;
		private boolean stop = false;
		private boolean firstIter = true;

		private ArrayList<AbstractBenchmark> benches = new ArrayList<AbstractBenchmark>();
		private ArrayList<Texture> textures = new ArrayList<Texture>();
		private Iterator<AbstractBenchmark> itty = null;
		private Iterator<Texture> titty = null;
		private AbstractBenchmark bench = null;
		private Texture loadingText = null;
		private Texture desc = null;
		private Texture startScreen = null;

		private ArrayList<BenchmarkResult> results = new ArrayList<BenchmarkResult>();

		public MyRenderer() {
			Config.maxPolysVisible = 500;
			Config.farPlane = 1500;
			Config.nearPlane = 1;
			Texture.defaultToKeepPixels(false);
		}

		public void stop() {
			stop = true;
			running = false;
			init = false;
		}

		public void onSurfaceChanged(GL10 gl, int w, int h) {
			Log.i("init", "width: " + w + "/height: " + h);
			if (fb != null) {
				fb.dispose();
				TextureManager.getInstance().flush();
			}

			if (gl20) {
				fb = new FrameBuffer(w, h);
			} else {
				fb = new FrameBuffer(gl, w, h);
			}

			resolution = "(" + w + "*" + h + ")";
			width = w;
			height = h;

			Resources res = getResources();

			benches.add(new DoubleDragon(res, world, fb, defaultTimeLong));
			benches.add(new ParticleSystem(res, world, fb, defaultTimeLong));
			benches.add(new SkeletalAnimations(res, world, fb, defaultTimeLong));
			benches.add(new ClothSimulation(res, world, fb, defaultTimeLong));
			benches.add(new Terrain(res, world, fb, defaultTimeLong * 1000));

			for (AbstractBenchmark bench : benches) {
				textures.add(bench.getDescription());
			}

			itty = benches.iterator();
			titty = textures.iterator();

			// Hack to avoid heap increases during runtime.

			int[] memoryAlloc = new int[1524 * 512 * 2]; // grow heap by ~4mb
			memoryAlloc[0] = 1; // use it
			memoryAlloc = null; // null it

			init = true;
			loaded = true;
		}

		public void onSurfaceCreated(GL10 gl, EGLConfig config) {
			if (!init) {

				TextureManager.getInstance().flush();

				world = new World();
				Resources res = getResources();

				Texture.defaultTo4bpp(!bit32);
				Config.useVBO = vbo;
				running = false;

				loadingText = new Texture(res.openRawResource(R.raw.loading));
				startScreen = new Texture(res.openRawResource(R.raw.startscreen));
			}
		}

		public void showResults() {
			myHandler.post(dialogOpener);
			// An3DBench.this.sendBroadcast(new Intent("testy"));
		}

		public void onDrawFrame(GL10 gl) {
			if (init) {
				if (running) {
					if (!stop) {
						if (firstIter) {
							firstIter = false;
							fb.clear(color);
							world.renderScene(fb);
							world.draw(fb);
							blitDesc();
						} else {

							if (bench == null && itty.hasNext()) {
								bench = itty.next();
								bench.init();
								itty.remove();
								if (desc != null) {
									TextureManager.getInstance().removeAndUnload(desc.toString(), fb);
									desc = null;
								}
								MemoryHelper.compact();
							}

							if (bench == null && !itty.hasNext()) {
								stop = true;
							}

							if (bench != null && !bench.done()) {
								boolean ret = bench.paint();
								fb.blit(bench.getInfoTexture(), 0, 0, 10, 10, 128, 16, true);
								if (!ret) {
									fb.clear(color);
									world.renderScene(fb);
									world.draw(fb);
									blitDesc();
									results.add(bench.getResult());
									bench = null;
								}
							} else {
								fb.clear(color);
								world.renderScene(fb);
								world.draw(fb);

								if (bench != null) {
									fb.blit(loadingText, 0, 0, 10, 10, 128, 16, false);
								} else {
									for (BenchmarkResult result : results) {
										Logger.log(result.toString());
									}

									benches.clear();
									world = null;
									MemoryHelper.compact();

									text = "";

									try {
										text += "Double dragon: " + results.get(0).getFps() + " fps\n";
										text += "Flower power: " + results.get(1).getFps() + " fps\n";
										text += "Ninjas' garden: " + results.get(2).getFps() + " fps\n";
										text += "Emperor's new clothes: " + results.get(3).getFps() + " fps\n";
										text += "Magic island: " + results.get(4).getFps() + " fps\n";

										BigDecimal score = results.get(0).getFps().multiply(new BigDecimal(20.00));
										score = score.add(results.get(1).getFps().multiply(new BigDecimal(20.00)));
										score = score.add(results.get(2).getFps().multiply(new BigDecimal(20.00)));
										score = score.add(results.get(3).getFps().multiply(new BigDecimal(20.00)));
										score = score.add(results.get(4).getFps().multiply(new BigDecimal(20.00)));
										score = score.multiply(new BigDecimal(8.658));
										text += "TOTAL SCORE: " + score.intValue();

										String urlPart = getDeviceData();

										int checkSum = 0;
										
										// Removed the chksum calculation to prevent manipulated uploads

										urlPart += "|" + checkSum;

										url += Base64.encodeBytes(urlPart.getBytes());
										Logger.log(url);
										Logger.log(text);

										// text += "SCORE: " + 1000;
									} catch (Exception e) {
										text = "Not a complete run!";
									}
								}
								showResults();
							}
						}
						fb.display();
					} else {
						try {
							fb.clear(color);
							fb.display();
							Thread.sleep(100);
						} catch (InterruptedException e) {
							e.printStackTrace();
						}
					}
				} else {
					// Show logo
					fb.clear(color);
					world.renderScene(fb);
					world.draw(fb);

					int xa = width / 2 - 128;
					int ya = height / 2 - 128;

					fb.blit(startScreen, 0, 0, xa, ya, 256, 256, true);
					fb.display();
					try {
						Thread.sleep(50);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				}
			}
		}

		private String getDeviceData() {
			StringBuilder sb = new StringBuilder();
			sb.append(Build.DEVICE.replace('|', '-')).append('|').append(Build.MODEL.replace('|', '-')).append('|').append(Build.PRODUCT.replace('|', '-')).append('|')
					.append(fb.getWidth()).append('*').append(fb.getHeight()).append('|').append(Build.VERSION.RELEASE);
			return sb.toString();
		}

		private void blitDesc() {
			if (desc == null && titty.hasNext()) {
				desc = titty.next();
				desc.enable4bpp(true);
				desc.setMipmap(false);
				TextureManager.getInstance().addTexture(desc.toString(), desc);
			}
			if (desc == null) {
				fb.blit(loadingText, 0, 0, 10, 10, 128, 16, false);
			} else {
				float mul = ((float) fb.getWidth()) / 480f;
				fb.blit(desc, 0, 0, 10, 10, 256, 128, (int) (256 * mul), (int) (128 * mul), -1, false, null);
			}
		}
	}

}