package com.threed.jpct.benchxl.parts;

import android.content.res.Resources;

import com.threed.jpct.Camera;
import com.threed.jpct.FrameBuffer;
import com.threed.jpct.IVertexController;
import com.threed.jpct.Light;
import com.threed.jpct.Logger;
import com.threed.jpct.Object3D;
import com.threed.jpct.Primitives;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.Texture;
import com.threed.jpct.TextureManager;
import com.threed.jpct.World;
import com.threed.jpct.benchxl.AbstractBenchmark;
import com.threed.jpct.benchxl.R;
import com.threed.jpct.benchxl.parts.impl.ClothController;

public class ClothSimulation extends AbstractBenchmark {

	private static final long serialVersionUID = 1L;
	private Texture text = null;
	private long lastTime = 0;

	private Object3D cloth1 = null;
	private ClothController clothSim1 = null;

	private Object3D sphere = null;
	private Texture backDrop=null;

	public ClothSimulation(Resources res, World world, FrameBuffer buffer, long maxTime) {
		super(res, world, buffer, maxTime);
		text = new Texture(ress.openRawResource(R.raw.cloth_txt));
	}

	@Override
	public Texture getInfoTexture() {
		return text;
	}

	@Override
	public void finishedPainting() {
		super.finishedPainting();
		if (!done) {
			if (lastTime != 0) {
				int ticks = Math.min(10, (int) (System.currentTimeMillis() - lastTime) / 10);
				if (ticks != 0) {
					lastTime = System.currentTimeMillis();
					//Logger.log(">Ticks: "+ticks);
					sphere.setScale(1);
					for (int i=0; i<ticks; i++) {
						cloth1.getMesh().applyVertexController();
					}
					cloth1.touch();
					
					if (runTime>10000) {
						sphere.translate(0, 3*ticks, 0);
						//clothSim1.setFreezeTop(false);
					}
				}
			} else {
				lastTime = System.currentTimeMillis();
			}
		}
	}

	@Override
	public void init() {
		super.init();

		world.setAmbientLight(100, 100, 100);
		
		Texture cloth1tex = new Texture(ress.openRawResource(R.raw.cloth_blue));
		TextureManager.getInstance().addTexture("cloth1", cloth1tex);
		
		Texture gold = new Texture(ress.openRawResource(R.raw.gold));
		TextureManager.getInstance().addTexture("gold", gold);
		
		backDrop = new Texture(ress.openRawResource(R.raw.background));
		TextureManager.getInstance().addTexture("back", backDrop);
		
		cloth1 = Primitives.getPlane(10, 50);
		cloth1.rotateX(-(float) Math.PI / 2);
		cloth1.rotateMesh();
		cloth1.clearRotation();
		cloth1.setTexture("cloth1");
		cloth1.setCulling(Object3D.CULLING_DISABLED);
		world.addObject(cloth1);
		cloth1.calcNormals();

		clothSim1 = new ClothController(world, true, 11, 11, 50f);
		cloth1.getMesh().setVertexController(clothSim1, IVertexController.PRESERVE_SOURCE_MESH);

		sphere = Primitives.getSphere(16, 150F);
		//sphere = Primitives.getCone(150);
		sphere.translate(0f, 375f, 0f);
		sphere.setTexture("gold");
		sphere.setCollisionMode(Object3D.COLLISION_CHECK_OTHERS);
		sphere.calcTextureWrapSpherical();
		world.addObject(sphere);
		sphere.setCollisionOptimization(Object3D.COLLISION_DETECTION_OPTIMIZED);

		Camera camera = world.getCamera();
		camera.setPosition(new SimpleVector(0, -50, -500));
		camera.lookAt(sphere.getTransformedCenter());
		camera.rotateX(0.3f);
		world.buildAllObjects();
		world.compileAllObjects(buffer);
		
		Light light=new Light(world);
		light.setIntensity(255, 100, 180);
		light.setPosition(new SimpleVector(150, -80, -100));

		Logger.log("Cloth simulation initialized!");
	}

	@Override
	public boolean paint() {
		if (!done && cloth1 != null) {
			sphere.setScale(0.95f);
			buffer.clear();
			buffer.blit(backDrop, 0, 0, 0, 0, 256, 256, buffer.getWidth(), buffer.getHeight(), -1, false, null);
			world.renderScene(buffer);
			world.draw(buffer);
		}

		return !done && running;
	}

	@Override
	public void dispose() {
		super.dispose();
		cloth1 = null;
		texMan.unloadTexture(buffer, text);
		texMan.removeAndUnload("cloth1", buffer);
		texMan.removeAndUnload("gold", buffer);
		texMan.removeAndUnload("back", buffer);
		world.removeAllObjects();
		world.removeAllLights();
		Logger.log("Cloth simulation done!");
	}
	
	@Override
	public Texture getDescription() {
		return new Texture(ress.openRawResource(R.raw.desc_cloth));
	}

}
