package com.threed.jpct.benchxl.parts;

import java.io.IOException;
import java.util.zip.ZipInputStream;

import android.content.res.Resources;

import com.threed.jpct.FrameBuffer;
import com.threed.jpct.Light;
import com.threed.jpct.Loader;
import com.threed.jpct.Logger;
import com.threed.jpct.Object3D;
import com.threed.jpct.Primitives;
import com.threed.jpct.RGBColor;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.Texture;
import com.threed.jpct.TextureManager;
import com.threed.jpct.World;
import com.threed.jpct.benchxl.AbstractBenchmark;
import com.threed.jpct.benchxl.R;

public class DoubleDragon extends AbstractBenchmark {

	private static final long serialVersionUID = 1L;
	private Texture text = null;
	private long lastTime = 0;

	private Object3D dragon = null;
	private Object3D dragon2 = null;
	private Object3D plane = null;
	private Object3D plane2 = null;

	public DoubleDragon(Resources res, World world, FrameBuffer buffer, long maxTime) {
		super(res, world, buffer, maxTime);
		text = new Texture(ress.openRawResource(R.raw.dragon_txt));
	}

	@Override
	public Texture getInfoTexture() {
		return text;
	}

	@Override
	public void finishedPainting() {
		super.finishedPainting();
		if (!done) {
			if (lastTime != 0) {
				int ticks = (int) (System.currentTimeMillis() - lastTime) / 10;
				if (ticks != 0) {
					dragon.rotateY(ticks * 0.005f);
					lastTime = System.currentTimeMillis();
				}
			} else {
				lastTime = System.currentTimeMillis();
			}
		}
	}

	@Override
	public void init() {
		super.init();

		world.setAmbientLight(0, 0, 0);

		Texture t = new Texture(ress.openRawResource(R.raw.marble));
		TextureManager.getInstance().addTexture("marble", t);
		
		t = new Texture(ress.openRawResource(R.raw.marbleouts));
		TextureManager.getInstance().addTexture("marble2", t);

		plane = Primitives.getPlane(4, 50);
		plane.rotateX((float) Math.PI / 2);
		plane.translate(0, -5, 0);
		plane.setTexture("marble");
		plane.setTransparency(8);
		plane.setAdditionalColor(RGBColor.WHITE);
		world.addObject(plane);
		
		plane2=new Object3D(plane, true);
		plane2.shareCompiledData(plane);
		plane2.setTexture("marble2");
		plane2.setTransparency(200);
		plane2.build();
		plane2.compile();
		world.addObject(plane2);

		plane.setSortOffset(100000);
		
		ZipInputStream zis = new ZipInputStream(ress.openRawResource(R.raw.dragon));
		try {
			zis.getNextEntry();
		} catch (IOException e) {
			throw new RuntimeException(e);
		}

		dragon = Loader.loadSerializedObject(zis);
		Light light = new Light(world);
		light.setIntensity(120, 255, 30);
		light.setPosition(new SimpleVector(0, -150, -20));

		dragon.build();

		dragon2 = new Object3D(dragon, true);
		dragon2.translate(0, 58.5f, 0);
		dragon2.getRotationMatrix().set(1, 1, -1);
		dragon2.invertCulling(true);
		dragon2.shareCompiledData(dragon);
		
		dragon.addChild(dragon2);

		world.addObject(dragon2);
		world.addObject(dragon);

		world.getCamera().setPosition(0, -100, -100);
		world.getCamera().lookAt(SimpleVector.ORIGIN);
		world.getCamera().rotateX(0.16f);

		world.buildAllObjects();
		world.compileAllObjects(buffer);
		
		Logger.log("Reflection initialized!");
	}

	@Override
	public boolean paint() {
		if (!done && dragon != null) {
			buffer.clear();
			world.renderScene(buffer);
			world.draw(buffer);
			/*
			try {
				Thread.sleep(2000);
			} catch(Exception e){}
			*/
		}

		return !done && running;
	}

	@Override
	public void dispose() {
		super.dispose();
		dragon = null;
		dragon2 = null;
		plane = null;
		texMan.unloadTexture(buffer, text);
		texMan.removeAndUnload("marble", buffer);
		texMan.removeAndUnload("marble2", buffer);
		world.removeAllObjects();
		world.removeAllLights();
		Logger.log("Cloth simulation done!");
	}

	@Override
	public Texture getDescription() {
		return new Texture(ress.openRawResource(R.raw.desc_dragon));
	}

}
