package com.threed.jpct.benchxl.parts;

import android.content.res.Resources;

import com.threed.jpct.Camera;
import com.threed.jpct.FrameBuffer;
import com.threed.jpct.Logger;
import com.threed.jpct.RGBColor;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.Texture;
import com.threed.jpct.TextureManager;
import com.threed.jpct.World;
import com.threed.jpct.benchxl.AbstractBenchmark;
import com.threed.jpct.benchxl.R;
import com.threed.jpct.benchxl.parts.impl.ParticleManager;

public class ParticleSystem extends AbstractBenchmark {

	private static final long serialVersionUID = 1L;
	private Texture text = null;
	private long lastTime = 0;

	private ParticleManager partMan = null;

	private Texture backDrop = null;

	private SimpleVector position = new SimpleVector(0, 0, 100);
	private SimpleVector dir = new SimpleVector(0, -.85, 0);

	public ParticleSystem(Resources res, World world, FrameBuffer buffer, long maxTime) {
		super(res, world, buffer, maxTime);
		text = new Texture(ress.openRawResource(R.raw.particles_txt));
	}

	@Override
	public Texture getInfoTexture() {
		return text;
	}

	@Override
	public void finishedPainting() {
		super.finishedPainting();
		if (!done) {
			if (lastTime != 0) {
				int ticks = Math.min(50, (int) (System.currentTimeMillis() - lastTime) / 20);
				if (ticks != 0) {
					lastTime = System.currentTimeMillis();

					partMan.addParticle(position, dir, 60, 5, false, 10, 1, RGBColor.WHITE, ticks * 2);
					partMan.move(ticks);
					
					if (runTime > 10000) {
					}
				}
			} else {
				lastTime = System.currentTimeMillis();
			}
		}
	}

	@Override
	public void init() {
		super.init();

		world.setAmbientLight(255, 255, 255);

		backDrop = new Texture(ress.openRawResource(R.raw.meadow));
		TextureManager.getInstance().addTexture("back", backDrop);

		Texture particle = new Texture(ress.openRawResource(R.raw.particle));
		TextureManager.getInstance().addTexture("particle1", particle);
		
		particle = new Texture(ress.openRawResource(R.raw.particle2));
		TextureManager.getInstance().addTexture("particle2", particle);
		
		particle = new Texture(ress.openRawResource(R.raw.particle3));
		TextureManager.getInstance().addTexture("particle3", particle);
		
		particle = new Texture(ress.openRawResource(R.raw.particle4));
		TextureManager.getInstance().addTexture("particle4", particle);

		partMan = new ParticleManager(world, 500);
		
		world.getCamera().moveCamera(Camera.CAMERA_MOVEUP, 60);
		world.getCamera().lookAt(position);

		Logger.log("Particle system initialized!");
	}

	@Override
	public boolean paint() {
		if (!done && partMan != null) {
			buffer.clear();
			buffer.blit(backDrop, 0, 0, 0, 0, 256, 256, buffer.getWidth(), buffer.getHeight(), -1, false, null);
			world.renderScene(buffer);
			world.draw(buffer);
		}

		return !done && running;
	}

	@Override
	public void dispose() {
		super.dispose();
		partMan = null;
		texMan.unloadTexture(buffer, text);
		texMan.removeAndUnload("back", buffer);
		texMan.removeAndUnload("particle1", buffer);
		texMan.removeAndUnload("particle2", buffer);
		texMan.removeAndUnload("particle3", buffer);
		texMan.removeAndUnload("particle4", buffer);
		world.removeAllObjects();
		world.removeAllLights();
		Logger.log("Particle system done!");
	}
	
	@Override
	public Texture getDescription() {
		return new Texture(ress.openRawResource(R.raw.desc_flower));
	}

}
