package com.threed.jpct.benchxl.parts;

import java.io.BufferedInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import raft.jpct.bones.Animated3D;
import raft.jpct.bones.AnimatedGroup;
import raft.jpct.bones.BonesIO;
import android.content.res.Resources;
import android.util.FloatMath;

import com.threed.jpct.Camera;
import com.threed.jpct.FrameBuffer;
import com.threed.jpct.GenericVertexController;
import com.threed.jpct.IVertexController;
import com.threed.jpct.Light;
import com.threed.jpct.Loader;
import com.threed.jpct.Logger;
import com.threed.jpct.Object3D;
import com.threed.jpct.Primitives;
import com.threed.jpct.RGBColor;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.Texture;
import com.threed.jpct.TextureInfo;
import com.threed.jpct.TextureManager;
import com.threed.jpct.World;
import com.threed.jpct.benchxl.AbstractBenchmark;
import com.threed.jpct.benchxl.R;

public class SkeletalAnimations extends AbstractBenchmark {

	private static final long serialVersionUID = 1L;
	private Texture text = null;
	private long lastTime = 0;

	private Ninja jim = null;
	private Object3D floor = null;
	private Texture backDrop;
	private float rot = 0;
	private float lastRot = 0;
	private SimpleVector pos = new SimpleVector();
	private SimpleVector tmp = new SimpleVector();
	private SimpleVector tmp2 = new SimpleVector();

	private List<Object3D> grasss = new ArrayList<Object3D>();

	public SkeletalAnimations(Resources res, World world, FrameBuffer buffer, long maxTime) {
		super(res, world, buffer, maxTime);
		text = new Texture(ress.openRawResource(R.raw.anim_txt));
	}

	@Override
	public Texture getInfoTexture() {
		return text;
	}

	@Override
	public void init() {
		super.init();
		world.setAmbientLight(100, 100, 100);

		Texture pt = new Texture(ress.openRawResource(R.raw.nskingr));
		texMan.addTexture("ninja", pt);

		Texture grass = new Texture(ress.openRawResource(R.raw.grass));
		texMan.addTexture("grass", grass);

		Texture grass2 = new Texture(ress.openRawResource(R.raw.grass2));
		texMan.addTexture("grass2", grass2);

		Texture blobb = new Texture(ress.openRawResource(R.raw.alpha), true);
		texMan.addTexture("blobb", blobb);

		backDrop = new Texture(ress.openRawResource(R.raw.gardenback));
		TextureManager.getInstance().addTexture("back", backDrop);

		jim = new Ninja(ress);
		jim.addToWorld(world);
		jim.scale(0.1f);
		pos = new SimpleVector(-55, 10, 0f);
		jim.translate(pos);

		floor = Primitives.getPlane(6, 40);
		floor.rotateX((float) Math.PI / 2f);

		TextureInfo ti = new TextureInfo(texMan.getTextureID("grass"));
		ti.add(texMan.getTextureID("blobb"), TextureInfo.MODE_MODULATE);

		floor.setTexture(ti);
		floor.translate(0, 9.5f, 0);
		floor.setTransparency(100);
		floor.setSortOffset(100000);
		world.addObject(floor);

		Object3D grassTmp = Loader.load3DS(ress.openRawResource(R.raw.grassmesh), 2)[0];
		grassTmp.setTexture("grass2");
		grassTmp.setTransparency(3);
		grassTmp.rotateZ((float) Math.PI);
		grassTmp.rotateMesh();
		grassTmp.clearRotation();
		grassTmp.build();

		Random rnd = new Random(102321312l);

		SimpleVector ps = new SimpleVector(-1, 0, 0f);
		for (int i = 0; i < 360; i += 18) {
			int start = 20 + (int) (rnd.nextFloat() * 5d);
			if (((i / 20) & 1) == 1) {
				start = 40 + (int) (rnd.nextFloat() * 15d);
			}
			for (int p = start; p < 95; p += 18) {
				ps.set(-1, 0, 0);
				ps.rotateY((float) Math.toRadians(i) + (0.15f - (float) rnd.nextFloat() * 0.3f));
				ps.scalarMul(p);
				Object3D ng = new Object3D(grassTmp, false);
				ng.setTransparency(4);
				ng.translate(ps);
				ng.translate(0, 10, 0);
				ng.setAdditionalColor(new RGBColor(200, 200, 200));
				ng.calcNormals();
				GrassController gc = new GrassController();
				ng.getMesh().setVertexController(gc, IVertexController.PRESERVE_SOURCE_MESH);
				ng.setUserObject(gc);
				ng.strip();
				ng.compile(true, true);
				ng.rotateY(((float) (i + p)) / 360f);
				grasss.add(ng);
				world.addObject(ng);
			}
		}
		
		Logger.log("Grass objects used: "+grasss.size());

		world.buildAllObjects();
		Light light = new Light(world);
		light.setIntensity(255, 100, 100);
		light.setPosition(new SimpleVector(50, -50, -10));

		Light light2 = new Light(world);
		light2.setIntensity(100, 100, 255);
		light2.setPosition(new SimpleVector(-50, -50, 10));

		world.compileAllObjects(buffer);

		Camera cam = world.getCamera();
		cam.setPosition(new SimpleVector(0, -30, -150));
		cam.rotateX(-0.06f);

		Logger.log("Animation initialized!");
	}

	private void processGrass(int ticks) {
		tmp2 = jim.getTranslation(tmp2);
		for (Object3D g : grasss) {
			GrassController gc = ((GrassController) g.getUserObject());
			tmp = g.getTranslation(tmp);
			tmp.y = 0;
			tmp2.y = 0;
			float dist = tmp.distance(tmp2);
			if (dist < 10) {
				gc.activate();
			} else {
				gc.update((float) ticks * 1.5f);
			}
			g.getMesh().applyVertexController();
			g.touch();
		}
	}

	@Override
	public void finishedPainting() {
		super.finishedPainting();
		if (lastTime != 0 && jim != null) {
			jim.animate(1);

			int ticks = (int) (System.currentTimeMillis() - lastTime) / 10;
			if (ticks != 0) {
				rot += ticks * 0.0075f;
				float delta = rot - lastRot;
				jim.rotateY(delta);
				lastRot = rot;

				pos.rotateY(-delta);
				jim.translate(pos);

				processGrass(ticks);

				lastTime = System.currentTimeMillis();
			}

		}
		lastTime = System.currentTimeMillis();
	}

	@Override
	public boolean paint() {
		if (!done && jim != null) {
			buffer.clear();
			buffer.blit(backDrop, 0, 0, 0, 0, 256, 256, buffer.getWidth(), buffer.getHeight(), -1, false, null);
			world.renderScene(buffer);
			world.draw(buffer);
		}

		return !done && running;
	}

	public void dispose() {
		super.dispose();
		jim = null;
		floor = null;
		texMan.unloadTexture(buffer, text);
		texMan.removeAndUnload("ninja", buffer);
		texMan.removeAndUnload("back", buffer);
		texMan.removeAndUnload("grass", buffer);
		texMan.removeAndUnload("grass2", buffer);
		texMan.removeAndUnload("blobb", buffer);
		texMan.removeTexture("grass.tga");
		world.removeAllObjects();
		world.removeAllLights();
		Logger.log("Animation done!");
	}

	private static class Ninja {

		private AnimatedGroup animatedGroup = null;
		private float index = 0;
		private long time = 0;

		public Ninja(Resources res) {
			init(res);
		}

		public SimpleVector getTranslation(SimpleVector into) {
			return animatedGroup.getRoot().getTranslation(into);
		}

		public void translate(SimpleVector trns) {
			animatedGroup.getRoot().clearTranslation();
			animatedGroup.getRoot().translate(trns);
		}

		public void scale(float scale) {
			animatedGroup.getRoot().setScale(scale);
		}

		public void rotateY(float rot) {
			animatedGroup.getRoot().rotateY(rot);
		}

		public void animate(int seq) {
			if (time == 0) {
				time = System.nanoTime() / 1000000L;
			}

			long dt = (System.nanoTime() / 1000000L) - time;
			time = dt + time;
			index += dt / 1000f;

			while (index > 1) {
				index -= 1;
			}
			animatedGroup.animateSkin(index, seq);
		}

		public void addToWorld(World world) {
			animatedGroup.addToWorld(world);
		}

		private void init(Resources res) {
			try {
				InputStream fis = new BufferedInputStream(res.openRawResource(R.raw.ninja), 256);
				try {
					AnimatedGroup skinnedGroup = BonesIO.loadGroup(fis);

					for (Animated3D o : skinnedGroup) {
						o.setTexture("ninja");
						o.build();
						o.discardMeshData();
						o.setVisibility(true);
					}
					animatedGroup = skinnedGroup;
				} finally {
					fis.close();
				}
			} catch (Exception e) {
				throw new RuntimeException(e);
			}
		}
	}

	private static class GrassController extends GenericVertexController {

		private static final long serialVersionUID = 1L;
		private float val = 100;
		private float degreeAdd = 0;
		private static float[] sins = new float[360 * 10];
		private static float[] coss = new float[360 * 10];

		static {
			for (int i = 0; i < sins.length; i++) {
				sins[i] = FloatMath.sin((float) Math.toRadians(((float) i) / 10f));
				coss[i] = FloatMath.cos((float) Math.toRadians(((float) i) / 10f));
			}
		}

		public boolean update(float inc) {

			degreeAdd += inc;
			degreeAdd %= 360;

			if (val > 100) {
				return false;
			}

			val += inc;
			if (val > 100) {
				val = 100;
			}
			return true;
		}

		public void activate() {
			val = 1f;
		}

		public void apply() {
			SimpleVector[] source = this.getSourceMesh();
			SimpleVector[] dest = this.getDestinationMesh();

			int end = source.length;

			if (val > 100) {
				for (int i = 0; i < end; i++) {
					SimpleVector s = source[i];
					if (s.y < -3) {
						SimpleVector d = dest[i];
						float sin = s.x < 0 ? sins[((int) (((degreeAdd - s.x) % 360) * 10)) % 3600] : sins[((int) (((degreeAdd + s.x) % 360) * 10)) % 3600];
						float cos = s.z < 0 ? coss[((int) (((degreeAdd - s.z) % 360) * 10)) % 3600] : sins[((int) (((degreeAdd + s.z) % 360) * 10)) % 3600];
						d.set(s.x + sin, s.y, s.z + cos);
					}
				}
				return;
			}

			float da = val / 100;

			for (int i = 0; i < end; i++) {
				SimpleVector s = source[i];
				SimpleVector d = dest[i];
				d.set(s.x + s.y * (1f - da), s.y * da, s.z - s.y * (1f - da));
			}

			if (val == 100) {
				val = 101;
			}
		}
	}
	
	@Override
	public Texture getDescription() {
		return new Texture(ress.openRawResource(R.raw.desc_ninja));
	}

}
