package com.threed.jpct.benchxl.parts;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipInputStream;

import android.content.res.Resources;
import android.util.FloatMath;

import com.threed.jpct.Camera;
import com.threed.jpct.FrameBuffer;
import com.threed.jpct.Light;
import com.threed.jpct.Loader;
import com.threed.jpct.Logger;
import com.threed.jpct.Matrix;
import com.threed.jpct.Object3D;
import com.threed.jpct.RGBColor;
import com.threed.jpct.SimpleVector;
import com.threed.jpct.Texture;
import com.threed.jpct.World;
import com.threed.jpct.benchxl.AbstractBenchmark;
import com.threed.jpct.benchxl.R;
import com.threed.jpct.util.LensFlare;
import com.threed.jpct.util.MemoryHelper;

public class Terrain extends AbstractBenchmark {

	private static final int TREES = 500;
	private static final long serialVersionUID = 1L;
	private Texture text = null;
	private long lastTime = 0;
	private RGBColor skyColor = new RGBColor(0, 0, 0);

	private Object3D terrain = null;

	private long time = System.currentTimeMillis();
	private int fps = 0;
	private SimpleVector moveDir = new SimpleVector(0, 0, 1);
	private LensFlare lens;
	private SimpleVector sunPos;
	private Object3D waterPlane;
	private float a;
	private Ticker ticker = new Ticker(20);
	private SimpleVector sunDir = new SimpleVector(0, -20000, 0);
	private Light sun;
	private SimpleVector tmp1 = new SimpleVector();
	private SimpleVector tmp2 = new SimpleVector();
	private SimpleVector tmp3 = new SimpleVector();
	private SimpleVector up = new SimpleVector(0, -1, 0);
	private float oldAng;
	private boolean renderLens = true;
	private ArrayList<SimpleVector> posList = new ArrayList<SimpleVector>();
	private ArrayList<Matrix> rotList = new ArrayList<Matrix>();
	private long fpsTime;
	private int pos = 0;
	private Object3D[] trees = null;
	private SimpleVector[] treePos=null;
	private boolean first=true;

	public Terrain(Resources res, World world, FrameBuffer buffer, long maxTime) {
		super(res, world, buffer, maxTime);
		text = new Texture(ress.openRawResource(R.raw.terrain_txt));
	}

	@Override
	public Texture getInfoTexture() {
		return text;
	}

	@Override
	public void init() {
		super.init();
		world.setAmbientLight(0, 0, 0);
		world.setClippingPlanes(1, 10000);

		Texture.defaultToMipmapping(true);
		Texture.defaultTo4bpp(false);

		Texture ground = new Texture(ress.openRawResource(R.raw.grass));
		texMan.addTexture("ground", ground);

		Texture water = new Texture(ress.openRawResource(R.raw.water3));
		texMan.addTexture("water_blue", water);

		Texture treet = new Texture(ress.openRawResource(R.raw.treet));
		texMan.addTexture("tree", treet);

		Logger.log("Loading terrain...");
		ZipInputStream zis = new ZipInputStream(ress.openRawResource(R.raw.terrain));
		try {
			zis.getNextEntry();
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		terrain = Loader.loadSerializedObject(zis);

		Logger.log("Setting up terrain...");
		terrain.setFixedPointMode(false);

		terrain.forceGeometryIndices(true);
		terrain.build();
		terrain.strip();
		world.addObject(terrain);

		world.setFogging(World.FOGGING_ENABLED);
		world.setFogParameters(7500, 10000, skyColor.getRed(), skyColor.getGreen(), skyColor.getBlue());

		sun = new Light(world);
		sun.setIntensity(0, 0, 0);

		sunPos = new SimpleVector(18904.664f, -1544.3248f, 39471.797f);
		sun.setPosition(sunPos);

		world.compileAllObjects(buffer);

		terrain.enableLazyTransformations();

		Logger.log("Loading water...");
		zis = new ZipInputStream(ress.openRawResource(R.raw.water));
		try {
			zis.getNextEntry();
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		waterPlane = Loader.loadSerializedObject(zis);
		waterPlane.scale(3.8f);
		waterPlane.rotateX((float) Math.PI / 2f);
		waterPlane.setTransparency(4);
		waterPlane.setSortOffset(-200000);
		world.addObject(waterPlane);
		waterPlane.forceGeometryIndices(true);
		waterPlane.setFixedPointMode(false);
		waterPlane.strip();
		waterPlane.translate(terrain.getCenter());
		waterPlane.translate(0, 1500, 0);
		waterPlane.build();
		waterPlane.compile();

		Camera cam = world.getCamera();
		cam.setPosition(17552.824f, -1755.2219f, 9189.418f);
		cam.rotateY(-0.58f);
		cam.rotateX(-0.17f);

		moveDir.rotateY(-0.58f);

		cam.setFOV(1.5f);

		Texture burst = new Texture(ress.openRawResource(R.raw.lens1), false);
		Texture halo1 = new Texture(ress.openRawResource(R.raw.lens2), false);
		Texture halo2 = new Texture(ress.openRawResource(R.raw.lens3), false);
		Texture halo3 = new Texture(ress.openRawResource(R.raw.lens4), false);

		texMan.addTexture("burst", burst);
		texMan.addTexture("halo1", halo1);
		texMan.addTexture("halo2", halo2);
		texMan.addTexture("halo3", halo3);

		lens = new LensFlare(SimpleVector.ORIGIN, "burst", "halo1", "halo2", "halo3");
		lens.setTransparency(5);
		lens.setGlobalScale(2 * buffer.getWidth() / 480);
		lens.setLightPosition(sunPos);

		loadPath();
		createTrees();
		
		cam = world.getCamera();
		cam.setPosition(posList.get(0));
		cam.getBack().setTo(rotList.get(0));
		processTrees();

		sunDir.rotateZ(-1.4f);
		
		Logger.log("Terrain initialized!");
		ticker.getTicks();
	}

	private void createTrees() {

		Object3D tree = Loader.loadSerializedObject(ress.openRawResource(R.raw.tree));
		tree.strip();
		tree.build();
		tree.setTransparency(40);
		tree.compile();

		List<Float> floats = readFloats(ress.openRawResource(R.raw.trees));
		int e = floats.size();
		treePos = new SimpleVector[e / 3];
		trees=new Object3D[TREES];
		int cnt = 0;
		for (int i = 0; i < e; i += 3) {
			SimpleVector pos = new SimpleVector(floats.get(i), floats.get(i + 1), floats.get(i + 2));
			treePos[cnt]=pos;
			cnt++;
		}
		
		e=trees.length;
		for (int i=0; i<e; i++) {
			Object3D t = new Object3D(tree, true);
			t.shareCompiledData(tree);
			t.strip();
			t.build();
			t.compile();
			t.setSortOffset(-250000);
			t.setVisibility(false);
			world.addObject(t);
			trees[i] = t;
		}
	}

	private void loadPath() {
		List<Float> floats = null;

		ZipInputStream zis = new ZipInputStream(ress.openRawResource(R.raw.terrainpath));
		try {
			zis.getNextEntry();
		} catch (IOException e) {
			throw new RuntimeException(e);
		}

		floats = readFloats(zis);

		int e = floats.size();
		for (int i = 0; i < e; i += 12) {
			SimpleVector pos = new SimpleVector(floats.get(i), floats.get(i + 1), floats.get(i + 2));
			Matrix rot = new Matrix();
			rot.set(0, 0, floats.get(i + 3));
			rot.set(1, 0, floats.get(i + 4));
			rot.set(2, 0, floats.get(i + 5));
			rot.set(0, 1, floats.get(i + 6));
			rot.set(1, 1, floats.get(i + 7));
			rot.set(2, 1, floats.get(i + 8));
			rot.set(0, 2, floats.get(i + 9));
			rot.set(1, 2, floats.get(i + 10));
			rot.set(2, 2, floats.get(i + 11));
			posList.add(pos);
			rotList.add(rot);
		}
	}

	@Override
	public void finishedPainting() {
		fps++;

		if (!done) {
			if (!running) {
				MemoryHelper.compact();
				try {
					Thread.sleep(700);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				startTime = System.currentTimeMillis();
			} else {
				frames++;
			}
			running = true;
			runTime = System.currentTimeMillis() - startTime;
		}

		if (lastTime != 0 && terrain != null) {
			int steps = (int) (System.currentTimeMillis() - lastTime) / 20;
			if (steps != 0) {
				if (pos + steps < posList.size()) {
					lastTime = System.currentTimeMillis();
					Camera cam = world.getCamera();
					pos += steps;
					cam.setPosition(posList.get(pos));
					cam.getBack().setTo(rotList.get(pos));
					
					processTrees();
					
					// Logger.log(posList.get(pos)+"  ---  "+rotList.get(pos)+" --- "+pos,
					// Logger.MESSAGE);
					
				} else {
					done = true;
					dispose();
				}
			}
		} else {
			lastTime = System.currentTimeMillis();
		}

		fpsTime = System.currentTimeMillis();

		if (fpsTime - time >= 1000) {
			//Logger.log(fps + "fps");
			time = fpsTime;
			fps = 0;
		}
	}

	private void processTrees() {
		// This is slow and culls too much at the border of the viewport.
		// It's not meant to be used in a productive application.

		int tp = 0;

		int end = treePos.length;
		int endt = trees.length;

		SimpleVector pos = world.getCamera().getPosition(tmp1);
		SimpleVector dir = world.getCamera().getDirection(tmp2);
		pos.scalarMul(-1);
		
		for (int i = 0; i < end; i++) {
			SimpleVector pp = treePos[i];
			SimpleVector toTree = tmp3;
			toTree.set(pp);
			toTree.add(pos);
			
			float len = toTree.length();
			if (len < 10000 && Math.abs(toTree.calcAngle(dir)) < Math.PI / 3.5f) {
				Object3D t = trees[tp];
				t.setVisibility(true);
				t.clearTranslation();
				t.translate(pp);
				t.clearRotation();
				t.rotateY(pp.x);
				if (len > 6000) {
					t.setTransparency((4000 - ((int) len - 6000)) / 100);
				} else {
					t.setTransparency(40);
				}
				tp++;
				if (tp >= endt) {
					break;
				}
			}
		}

		for (int i = tp; i < endt; i++) {
			Object3D tree = trees[i];
			tree.setVisibility(false);
		}
	}
	@Override
	public boolean paint() {
		if (!done && terrain != null) {
			int ticks = ticker.getTicks();

			if (first) {
				ticks=1;
				first=false;
			}
			
			if (ticks > 0) {
				sunDir.rotateZ(ticks * 0.0005f);

				SimpleVector sunp = tmp1;
				sunp.set(sunPos);
				sunp.add(sunDir);
				sun.setPosition(sunp);
				lens.setLightPosition(sunp);

				sunp.x -= sunPos.x;
				sunp.y -= sunPos.y;
				sunp.z -= sunPos.z;

				sunp = sunp.calcSub(sunPos);
				SimpleVector sn = sunp.normalize(tmp2);
				float ang = sn.calcAngle(up);

				if (ang != oldAng) {
					oldAng = ang;
					float cos = FloatMath.cos(ang);
					if (cos >= 0) {
						float sqrt = FloatMath.sqrt(cos);
						world.setAmbientLight((int) (20f * sqrt + 10), (int) (20f * cos + 10), (int) (20f * cos + 10));
						sun.setIntensity((int) (240f * sqrt + 10), (int) (240f * cos + 10), (int) (240f * cos + 10));
						renderLens = true;
						skyColor.setTo((int) (195f * FloatMath.sqrt(sqrt)), (int) (210f * sqrt), (int) (230f * sqrt), 0);
						world.setFogParameters(7500, 10000, skyColor.getRed(), skyColor.getGreen(), skyColor.getBlue());
					} else {
						renderLens = false;
						sunDir.rotateZ(ticks * 0.02f);
					}
				}

				lens.update(buffer, world);
				// world.getCamera().moveCamera(moveDir, 2*ticks);

				float sina=FloatMath.sin(a);
				waterPlane.translate(sina, sina / 20f, FloatMath.cos(a));
				a += ticks * 0.01f;
			}
			
			buffer.clear(skyColor);

			world.renderScene(buffer);
			world.draw(buffer);

			if (renderLens) {
				lens.render(buffer);
			}
		}

		return !done && running;
	}

	public void dispose() {
		super.dispose();
		terrain = null;
		waterPlane = null;
		texMan.unloadTexture(buffer, text);
		texMan.removeAndUnload("ground", buffer);
		texMan.removeAndUnload("water_blue", buffer);
		texMan.removeAndUnload("tree", buffer);
		texMan.removeAndUnload("burst", buffer);
		texMan.removeAndUnload("halo1", buffer);
		texMan.removeAndUnload("halo2", buffer);
		texMan.removeAndUnload("halo3", buffer);
		world.removeAllObjects();
		world.removeAllLights();
		Logger.log("Terrain done!");
	}

	private static class Ticker {

		private int rate;
		private long s2;

		public static long getTime() {
			return System.currentTimeMillis();
		}

		public Ticker(int tickrateMS) {
			rate = tickrateMS;
			s2 = Ticker.getTime();
		}

		public int getTicks() {
			long i = Ticker.getTime();
			if (i - s2 > rate) {
				int ticks = (int) ((i - s2) / (long) rate);
				s2 += (long) rate * ticks;
				return ticks;
			}
			return 0;
		}
	}
	
	@Override
	public Texture getDescription() {
		return new Texture(ress.openRawResource(R.raw.desc_island));
	}

}
